local textureList = {
	["Rectangular"] = {
		"BG",
	},
	["Horizontal"] = {
		"XpMid",
		"XpL",
		"XpR",
		"MicroBGL",
		"MicroBGR",
		"MicroBGMid",
		"ButtonBGL",
		"ButtonBGR",
		"ButtonBGMid",
		"BagBGL",
		"BagBGR",
		"BagBGMid",
		"PageBG",
		"PageOverlay",
		"PageButtonBG",
		"PageMarker",
		"PageUpUp",
		"PageUpDown",
		"PageUpHighlight",
		"PageDownUp",
		"PageDownDown",
		"PageDownHighlight",
		"HealthBarBG",
		"HealthBarOverlay",
		"PowerBarBG",
		"PowerBarOverlay",
		"Divider1",
		"Divider2",
		"Divider3",
		"Boarder",
		"EndCapL",
		"EndCapR",
	},
}

function AltActionBar_OnLoad(self)
	--Setup the XP bar
	for i=1,19 do
		local texture = self.xpBar:CreateTexture("AltActionBarXpDiv"..i, "OVERLAY", nil, 2)
		local textureList = textureList["Horizontal"]
		texture:SetSize(7, 14)
		texture:SetTexCoord(0.2773438, 0.2910156, 0.390625, 0.4179688)
		self.xpBar["XpDiv"..i] = texture
		self["XpDiv"..i] = texture
		textureList[#textureList + 1] = "XpDiv"..i
	end
	self.textureList = textureList
		
	--Add PageUp button Textures
	self["PageUpUp"] = self.PageUpButton:GetNormalTexture()
	self["PageUpDown"] = self.PageUpButton:GetPushedTexture()
	self["PageUpHighlight"] = self.PageUpButton:GetHighlightTexture()
	
	--Add PageDown button Textures
	self["PageDownUp"] = self.PageDownButton:GetNormalTexture()
	self["PageDownDown"] = self.PageDownButton:GetPushedTexture()
	self["PageDownHighlight"] = self.PageDownButton:GetHighlightTexture()
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("PLAYER_XP_UPDATE")
	self:RegisterEvent("UPDATE_FACTION")
	self:RegisterEvent("ACTIONBAR_PAGE_CHANGED")
	
	hooksecurefunc("UIParent_ManageFramePosition", AltActionBar_MovePetActionButton)
	hooksecurefunc("MoveMicroButtons", AltActionBar_MoveMicroButtons)
	hooksecurefunc("CastingBarFrame_SetLook", AltActionBar_CastingBar_SetLook)
	
	for k, v in pairs(textureList) do
		for _, v in pairs(v) do
			if k == "Rectangular" then
				self[v]:SetTexture("Interface\\PlayerActionBarAlt\\Darkmoon", self[v]:GetHorizTile())
			else
				self[v]:SetTexture("Interface\\PlayerActionBarAlt\\Mechanical", self[v]:GetHorizTile())
			end
		end
	end
end

function AltActionBar_OnEvent(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		AltActionBar_SetState()
		AltActionBar_SetScale()
		AltActionBar_StanceBar_SetScale()
		AltActionBar_PetBar_SetScale()
		AltActionBar_CastingBar_SetState()
		AltActionBar_StatusBars_Update()
		AltActionBar_SetPageValue(GetActionBarPage())
	elseif event == "PLAYER_LEVEL_UP" then
		AltActionBar_XpBar_Update()
	elseif event == "PLAYER_XP_UPDATE" then
		AltActionBar_XpBar_Update()
	elseif event == "UPDATE_FACTION" then
		AltActionBar_XpBar_Update()
	elseif event == "ACTIONBAR_PAGE_CHANGED" then
		AltActionBar_SetPageValue(GetActionBarPage())
	end
end

function AltActionBar_CalcSize()
	local width, xpWidth, anchor, buttonAnchor
	local hasPage = AltUISVPC["Action Bar"]["Reforge"]["Paging Bar"]
	local hasBag = AltUISVPC["Action Bar"]["Reforge"]["Bag Slots"]
	local hasStatus = AltUISVPC["Action Bar"]["Reforge"]["Status Bars"]
	local value = AltUISVPC["Action Bar"]["Resize"]
	local size = ( 12 - value/2 ) * 42
	
	AltActionBar.pageFrame:Hide()
	AltActionBar.bagFrame:Hide()
	AltActionBar.healthFrame:Hide()
	AltActionBar.powerFrame:Hide()
	if hasPage and hasBag then
		width, xpWidth, anchor, buttonAnchor  = 1185, 900, 174, -326
		AltActionBar.bagFrame:Show()
		AltActionBar.pageFrame:Show()
	elseif hasPage then
		width, xpWidth, anchor, buttonAnchor = 1090, 805, 220, -280
		AltActionBar.pageFrame:Show()
	elseif hasBag then
		width, xpWidth, anchor, buttonAnchor = 1100, 815, 132, -368
		AltActionBar.bagFrame:Show()
	else
		width, xpWidth, anchor, buttonAnchor = 1005, 720, 178, -322
	end
	if hasStatus then
		AltActionBar.healthFrame:Show()
		AltActionBar.powerFrame:Show()
	end
	width, xpWidth, anchor, buttonAnchor = width-size, xpWidth-size, anchor-size/2, buttonAnchor+size/2
	AltActionBar.ButtonBGMid:SetWidth(490-size)
	ActionButton1:ClearAllPoints()
	ActionButton1:SetPoint("BOTTOM", AltActionBar, "BOTTOM", buttonAnchor, 3)
	AltActionBar.PageUpButton:SetPoint("BOTTOM", AltActionBar, buttonAnchor-92, 44)
	AltActionBar.PageDownButton:SetPoint("BOTTOM", AltActionBar, buttonAnchor-92, 8)
	AltActionBar.Divider2:SetPoint("BOTTOM", anchor, 0)
	AltActionBar:SetWidth(width)	
	AltActionBar_XpBar_SetWidth(xpWidth)
	AltActionBar_Reposition()
end

function AltActionBar_Reposition()
	local isClassic = AltUISVPC["Action Bar"]["Classical"]["Action Bar"]
	local isStacked = AltUISVPC["Action Bar"]["Reposition"]
	local value = AltUISVPC["Action Bar"]["Resize"]
	for i=1, 12 do
		local topButton = _G["MultiBarBottomLeftButton"..i]
		topButton:SetAlpha(1)
		topButton:EnableMouse(true)
		
		local bottomButton = _G["ActionButton"..i]
		bottomButton:SetAlpha(1)
		bottomButton:EnableMouse(true)
		
		if not isClassic then
			if i > 6 and not isStacked then
				bottomButton = _G["MultiBarBottomLeftButton"..(i-6)]
			end
			if i > value/2 then
				topButton:SetAlpha(0)
				topButton:EnableMouse(false)
				bottomButton:SetAlpha(0)
				bottomButton:EnableMouse(false)
			else
				topButton:SetAlpha(1)
				topButton:EnableMouse(true)
				bottomButton:SetAlpha(1)
				bottomButton:EnableMouse(true)
			end
		end
	end
	AltActionBar_MoveActionButtons()
end

function AltActionBar_SetPageValue(page)
	if page == 6 then
		AltActionBar.pageFrame.PageMarker:Hide()
	else
		AltActionBar.pageFrame.PageMarker:Show()
	end
	PlaySound("UChatScrollButton")
	AltActionBar.pageFrame.PageMarker:SetPoint("CENTER", AltActionBar.pageFrame.PageOverlay, "BOTTOM", 0, page/4 * (AltActionBar.pageFrame.PageOverlay:GetHeight() - 35))
end

local AltActionBarBackpackButton_SetChecked = MainMenuBarBackpackButton.SetChecked
function MainMenuBarBackpackButton.SetChecked(self, checked)
	AltActionBarBackpackButton_SetChecked(self, checked)
	if MainMenuBarBackpackButton:GetChecked() then
		AltActionBarBackpackButton:SetButtonState("PUSHED", 1)
	else
		AltActionBarBackpackButton:SetButtonState("NORMAL")
	end
end

function AltActionBar_MoveBagButtons()
	CharacterBag0Slot:ClearAllPoints()
	CharacterBag2Slot:ClearAllPoints()
	if AltActionBar:IsShown() then
		CharacterBag0Slot:SetPoint("BOTTOMRIGHT", AltActionBar.BagBGR, "BOTTOMRIGHT", -11, 10)
		CharacterBag2Slot:SetPoint("BOTTOM", CharacterBag0Slot, "TOP", 0, 2)
		for i=0, 3 do
			_G["CharacterBag"..i.."Slot"]:SetParent(AltActionBar.bagFrame)
		end
	else
		CharacterBag0Slot:SetPoint("RIGHT", MainMenuBarBackpackButton, "LEFT", -2, 0)
		CharacterBag2Slot:SetPoint("RIGHT", CharacterBag1Slot, "LEFT", -2, 0)
		for i=0, 3 do
			_G["CharacterBag"..i.."Slot"]:SetParent(MainMenuBarArtFrame)
		end
	end
end

function AltActionBar_MoveMicroButtons(anchor, achorTo, relAnchor, x, y, isStacked)
	if not isStacked then
		PVPMicroButton:ClearAllPoints()
		CharacterMicroButton:ClearAllPoints()
		if AltActionBar:IsShown() then
			CharacterMicroButton:SetPoint("TOPLEFT", AltActionBar.MicroBGL, "TOPLEFT", 11, 15)
			PVPMicroButton:SetPoint("TOPLEFT", CharacterMicroButton, "BOTTOMLEFT", 0, 24)
		else
			CharacterMicroButton:SetPoint("BOTTOMLEFT", MainMenuBarArtFrame, "BOTTOMLEFT", 548, 2)
			PVPMicroButton:SetPoint("BOTTOMLEFT", GuildMicroButton, "BOTTOMRIGHT", -3, 0)
		end
	end
end

function AltActionBar_MoveActionButtons()
	MultiBarBottomLeftButton1:ClearAllPoints()
	MultiBarBottomRightButton1:ClearAllPoints()
	MultiBarBottomLeftButton7:ClearAllPoints()
	ActionButton7:ClearAllPoints()
	if AltActionBar:IsShown() then
		local isStacked = AltUISVPC["Action Bar"]["Reposition"]
		if not isStacked then
			ActionButton7:SetPoint("BOTTOM", ActionButton1, "TOP", 0, 6)
			MultiBarBottomLeftButton1:SetPoint("LEFT", ActionButton6, "RIGHT", 6, 0)
			MultiBarBottomLeftButton7:SetPoint("BOTTOM", MultiBarBottomLeftButton1, "TOP", 0, 6)
		else
			ActionButton7:SetPoint("LEFT", ActionButton6, "RIGHT", 6, 0)
			MultiBarBottomLeftButton1:SetPoint("BOTTOM", ActionButton1, "TOP", 0, 6)
			MultiBarBottomLeftButton7:SetPoint("LEFT", MultiBarBottomLeftButton6, "RIGHT", 6, 0)
		end
		MultiBarBottomRightButton1:SetPoint("BOTTOMRIGHT", AltActionBar, "BOTTOMRIGHT", -606, 125)
		PossessButton1:SetPoint("BOTTOMLEFT", AltActionBar, 145, 125)
		StanceButton1:SetPoint("BOTTOMLEFT", AltActionBar, 145, 125)
	else
		MultiBarBottomLeftButton1:SetPoint("BOTTOMLEFT", MultiBarBottomLeft, 0, 0)
		MultiBarBottomRightButton1:SetPoint("BOTTOMLEFT", MultiBarBottomRight, 0, 0)
		MultiBarBottomLeftButton7:SetPoint("LEFT", MultiBarBottomLeftButton6, "RIGHT", 6, 0)
		ActionButton7:SetPoint("LEFT", ActionButton6, "RIGHT", 6, 0)
		PossessButton1:SetPoint("BOTTOMLEFT", PossessBarFrame, 10, 3)
		StanceButton1:SetPoint("BOTTOMLEFT", StanceBarFrame, 10, 3)
		ActionButton1:ClearAllPoints()
		ActionButton1:SetPoint("BOTTOMLEFT", MainMenuBarArtFrame, 8, 4)
	end
end

function AltActionBar_MovePetActionButton()
	if not InCombatLockdown() then
		PetActionButton1:ClearAllPoints()
		if AltActionBar:IsShown() then
			if MultiBarBottomRight:IsShown() then
				PetActionButton1:SetPoint("BOTTOMRIGHT", AltActionBar, "BOTTOMRIGHT", -548, 169)
			else
				PetActionButton1:SetPoint("BOTTOMRIGHT", AltActionBar, "BOTTOMRIGHT", -485, 125)
			end
		else
			PetActionButton1:SetPoint("BOTTOMLEFT", PetActionBarFrame, 36, 2)
		end
	end
end

function AltActionBar_CastingBar_SetLook(castBar, look)
	local isClassic = AltUISVPC["Action Bar"]["Classical"]["Casting Bar"]
	if look == "ALTCLASSIC" then
		castBar.border:SetTexture("Interface\\CastingBar\\UI-CastingBar-Border")
		castBar.border:SetPoint("TOP", 0, 26)
		castBar.borderShield:SetPoint("TOP", 0, 26)
		castBar.text:SetPoint("TOP", 0, 3)
		castBar.icon:Hide()
		castBar.barSpark.offsetY = 0
		castBar.barFlash:SetTexture("Interface\\CastingBar\\UI-CastingBar-Flash")
		castBar.barFlash:SetPoint("TOP", 0, 26)
	elseif look == "ALTUNITFRAME" or look == "CLASSIC" and not isClassic then
		castBar.border:SetTexture("Interface\\CastingBar\\UI-CastingBar-Border-Small")
		castBar.border:SetPoint("TOP", 0, 26)
		castBar.borderShield:SetPoint("TOP", -6, 26)
		castBar.text:SetPoint("TOP", 0, 3)
		castBar.icon:Show()
		castBar.icon:SetSize(24, 24)
		castBar.barSpark.offsetY = 0
		castBar.barFlash:SetTexture("Interface\\CastingBar\\UI-CastingBar-Flash-Small")
		castBar.barFlash:SetPoint("TOP", 0, 26)
	end
	AltActionBar_CastingBar_SetScale(look)
end

function AltActionBar_CastingBar_SetState()
	local isClassic = AltUISVPC["Action Bar"]["Classical"]["Casting Bar"]
	local bottomY = CastingBarFrame:GetBottom()
	local screenWidth, screenHeight = GetScreenWidth(), GetScreenHeight()
	if bottomY and bottomY < screenHeight / 2 then
		if isClassic then
			AltActionBar_CastingBar_SetLook(CastingBarFrame, "ALTCLASSIC")
		else
			AltActionBar_CastingBar_SetLook(CastingBarFrame, "ALTUNITFRAME")
		end
	end
end

function AltActionBar_CastingBar_SetScale(look)
	local bottomY = CastingBarFrame:GetBottom()
	local screenWidth, screenHeight = GetScreenWidth(), GetScreenHeight()
	if bottomY and bottomY < screenHeight / 2 or look == "CLASSIC" then
		local actionBarScale = AltUISVPC["Action Bar"]["Rescale"]
		CastingBarFrame:SetScale(actionBarScale)
		CastingBarFrame.icon:SetSize(24, 24)
	else
		local unitFrameScale = AltUISVPC["Unit Frames"]["Rescale"]
		CastingBarFrame:SetScale(unitFrameScale)
		CastingBarFrame.icon:SetSize(16, 16)
	end
end

function AltActionBar_PetBar_SetScale()
	local hasPet = AltUISVPC["Action Bar"]["Reforge"]["Pet Bar"]
	if hasPet then
		PetActionBarFrame:SetScale(1)
	else
		PetActionBarFrame:SetScale(0.01)
	end
end

function AltActionBar_StanceBar_SetScale()
	local hasStance = AltUISVPC["Action Bar"]["Reforge"]["Stance Bar"]
	if hasStance then
		StanceBarFrame:SetScale(1)
	else
		StanceBarFrame:SetScale(0.01)
	end
end

function AltActionBar_SetState()
	local isClassic = AltUISVPC["Action Bar"]["Classical"]["Action Bar"]
	if isClassic and AltActionBar:IsShown() then
		AltActionBar_SetClassic()
	elseif not isClassic then
		if not AltActionBar:IsShown() then
			AltActionBar_SetAlt()
		end
		AltActionBar_CalcSize()
	end
end

function AltActionBar_SetScale()
	local scale = AltUISVPC["Action Bar"]["Rescale"]
	MainMenuBar:SetScale(scale)
	AltActionBar_CastingBar_SetScale()
end

function AltActionBar_SetAlt()
	UIPARENT_MANAGED_FRAME_POSITIONS["ExtraActionBarFrame"].yOffset = 80
	UIPARENT_MANAGED_FRAME_POSITIONS["CastingBarFrame"].yOffset = 80
	MainMenuBarBackpackButton:SetAlpha(0)
	MainMenuBarBackpackButton:EnableMouse(false)
	ActionBarUpButton:SetAlpha(0)
	ActionBarUpButton:EnableMouse(false)
	ActionBarDownButton:SetAlpha(0)
	ActionBarDownButton:EnableMouse(false)
	MainMenuExpBar:SetAlpha(0)
	MainMenuExpBar:EnableMouse(false)
	ReputationWatchBar:SetAlpha(0)
	ReputationWatchBar:EnableMouse(false)
	MainMenuBarMaxLevelBar:SetAlpha(0)
	MainMenuBarMaxLevelBar:EnableMouse(false)
	MainMenuBarTexture0:SetAlpha(0)
	MainMenuBarTexture1:SetAlpha(0)
	MainMenuBarTexture2:SetAlpha(0)
	MainMenuBarTexture3:SetAlpha(0)
	MainMenuBarLeftEndCap:SetAlpha(0)
	MainMenuBarRightEndCap:SetAlpha(0)
	MainMenuBarPageNumber:SetAlpha(0)
	StanceBarLeft:SetAlpha(0)
	StanceBarMiddle:SetAlpha(0)
	StanceBarRight:SetAlpha(0)
	SlidingActionBarTexture0:SetAlpha(0)
	SlidingActionBarTexture1:SetAlpha(0)

	AltActionBar:Show()
	AltActionBar_MoveBagButtons()
	AltActionBar_MoveMicroButtons()
	AltActionBar_CalcSize()
	UIParent_ManageFramePositions()
end

function AltActionBar_SetClassic()
	UIPARENT_MANAGED_FRAME_POSITIONS["ExtraActionBarFrame"].yOffset = 40
	UIPARENT_MANAGED_FRAME_POSITIONS["CastingBarFrame"].yOffset = 40
	MainMenuBarBackpackButton:SetAlpha(1)
	MainMenuBarBackpackButton:EnableMouse(true)
	ActionBarUpButton:SetAlpha(1)
	ActionBarUpButton:EnableMouse(true)
	ActionBarDownButton:SetAlpha(1)
	ActionBarDownButton:EnableMouse(true)
	MainMenuExpBar:SetAlpha(1)
	MainMenuExpBar:EnableMouse(true)
	ReputationWatchBar:SetAlpha(1)
	ReputationWatchBar:EnableMouse(true)
	MainMenuBarMaxLevelBar:SetAlpha(1)
	MainMenuBarMaxLevelBar:EnableMouse(true)
	MainMenuBarTexture0:SetAlpha(1)
	MainMenuBarTexture1:SetAlpha(1)
	MainMenuBarTexture2:SetAlpha(1)
	MainMenuBarTexture3:SetAlpha(1)
	MainMenuBarLeftEndCap:SetAlpha(1)
	MainMenuBarRightEndCap:SetAlpha(1)
	MainMenuBarPageNumber:SetAlpha(1)
	StanceBarLeft:SetAlpha(1)
	StanceBarMiddle:SetAlpha(1)
	StanceBarRight:SetAlpha(1)
	SlidingActionBarTexture0:SetAlpha(1)
	SlidingActionBarTexture1:SetAlpha(1)

	AltActionBar:Hide()
	AltActionBar_MoveBagButtons()
	AltActionBar_MoveMicroButtons()
	AltActionBar_Reposition()
	UIParent_ManageFramePositions()
end

function AltActionBar_StatusBars_ShowTooltip(self)
	if GetMouseFocus() == self then
		local value = self:GetValue()
		local _, valueMax = self:GetMinMaxValues()
		if valueMax > 0 then
			local exhaustionStateID, exhaustionStateName, exhaustionStateMultiplier = GetRestState()
			local text = format("%s/%s (%s%%)", AbbreviateLargeNumbers(value), AbbreviateLargeNumbers(valueMax), tostring(math.ceil((value / valueMax) * 100)))
			GameTooltip:SetOwner(self, self.tooltipAnchorPoint)
			if self.prefix then
				GameTooltip:AddLine(self.prefix)
			end
			GameTooltip:AddLine(text, 1.0,1.0,1.0)
			if self.prefix == XP then
				GameTooltip:AddLine("\n"..format(EXHAUST_TOOLTIP1, exhaustionStateName, exhaustionStateMultiplier * 100))
			end
			GameTooltip:Show()
		end
	end
end

function AltActionBar_StatusBars_Update()
	UnitFrameHealthBar_Update(AltActionBar.healthFrame.healthBar, "player")
	UnitFrameManaBar_Update(AltActionBar.powerFrame.powerBar, "player")
end

function AltActionBar_XpBar_SetWidth(width)
	AltActionBar.xpBar.XpMid:SetWidth(width)
	AltActionBar.xpBar:SetWidth(width+16)
	local divWidth = (width+16)/20
	local xpos = divWidth-10
	for i=1,19 do
		local texture = AltActionBar.xpBar["XpDiv"..i]
		texture:SetPoint("LEFT", AltActionBar.xpBar.XpMid, "LEFT", floor(xpos), 10)
		xpos = xpos + divWidth
	end
end

function AltActionBar_XpBar_Update()
	if ReputationWatchBar:IsShown() then
		AltActionBar.xpBar:SetMinMaxValues(ReputationWatchStatusBar:GetMinMaxValues())
		AltActionBar.xpBar:SetValue(ReputationWatchStatusBar:GetValue())
		AltActionBar.xpBar:SetStatusBarColor(ReputationWatchStatusBar:GetStatusBarColor())
		SetTextStatusBarTextPrefix(AltActionBar.xpBar, GetWatchedFactionInfo())
	elseif MainMenuExpBar:IsShown() then
		AltActionBar.xpBar:SetMinMaxValues(MainMenuExpBar:GetMinMaxValues())
		AltActionBar.xpBar:SetValue(MainMenuExpBar:GetValue())
		AltActionBar.xpBar:SetStatusBarColor(MainMenuExpBar:GetStatusBarColor())
		SetTextStatusBarTextPrefix(AltActionBar.xpBar, XP)
	else
		AltActionBar.xpBar:SetMinMaxValues(0,0)
		AltActionBar.xpBar:SetValue(0)
		AltActionBar.xpBar:SetStatusBarColor(0, 0, 0, 0)
		SetTextStatusBarTextPrefix(AltActionBar.xpBar, "")
	end
end